import torch
from torch.utils.data import Dataset
import torchaudio
import os

class AudioDataset(Dataset):
    def __init__(self, input_dir, target_duration=None, padding_mode='constant', padding_value=0.0):
        self.input_files = [os.path.join(input_dir, f) for f in os.listdir(input_dir) if f.endswith('.wav')]
        self.target_duration = target_duration  # Duration in seconds or None if not set
        self.padding_mode = padding_mode
        self.padding_value = padding_value

    def __len__(self):
        return len(self.input_files)


    def __getitem__(self, idx):
        high_quality_wav, sr_original = torchaudio.load(self.input_files[idx], normalize=True)

        resample_transform = torchaudio.transforms.Resample(sr_original, 16000)
        low_quality_wav = resample_transform(high_quality_wav)

        # Calculate target length based on desired duration and 16000 Hz
        if self.target_duration is not None:
            target_length = int(self.target_duration * 44100)
        else:
            # Calculate duration of original high quality audio
            duration_original = high_quality_wav.shape[1] / sr_original
            target_length = int(duration_original * 16000)

        # Pad both to the calculated target length
        high_quality_wav = self.pad_tensor(high_quality_wav, target_length)
        low_quality_wav = self.pad_tensor(low_quality_wav, target_length)

        return low_quality_wav, high_quality_wav

    def pad_tensor(self, tensor, target_length):
        current_length = tensor.size(1)
        if current_length < target_length:
            padding_amount = target_length - current_length
            padding = (0, padding_amount)
            tensor = torch.nn.functional.pad(tensor, padding, mode=self.padding_mode, value=self.padding_value)
        elif current_length > target_length:
            tensor = tensor[:, :target_length]
        return tensor
