import torch
import torch.nn as nn
import torch.optim as optim
import torchaudio
import tqdm

from torch.utils.data import random_split
from torch.utils.data import DataLoader

from data import AudioDataset
from generator import SISUGenerator
from discriminator import SISUDiscriminator

# Check for CUDA availability
device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
print(f"Using device: {device}")

# Initialize dataset and dataloader
dataset_dir = './dataset/good'
dataset = AudioDataset(dataset_dir, target_duration=2.0)  # 5 seconds target duration

dataset_size = len(dataset)
train_size = int(dataset_size * .9)
val_size = int(dataset_size-train_size)

train_dataset, val_dataset = random_split(dataset, [train_size, val_size])

train_data_loader = DataLoader(train_dataset, batch_size=4, shuffle=True)
val_data_loader = DataLoader(val_dataset, batch_size=4, shuffle=True)

# Initialize models and move them to device
generator = SISUGenerator()
discriminator = SISUDiscriminator()

generator = generator.to(device)
discriminator = discriminator.to(device)

# Loss and optimizers
criterion = nn.MSELoss()  # Use Mean Squared Error loss
optimizer_g = optim.Adam(generator.parameters(), lr=0.0005, betas=(0.5, 0.999))
optimizer_d = optim.Adam(discriminator.parameters(), lr=0.0001, betas=(0.5, 0.999))

# Learning rate scheduler
scheduler = torch.optim.lr_scheduler.ReduceLROnPlateau(optimizer_d, mode='min', factor=0.1, patience=5)

# Training loop
num_epochs = 500
for epoch in range(num_epochs):
    original, crap_audio = torch.empty((1,2,3)), torch.empty((1,2,3))
    for low_quality, high_quality in tqdm.tqdm(train_data_loader):
        low_quality = low_quality.to(device)
        high_quality = high_quality.to(device)
        batch_size = low_quality.size(0)
        real_labels = torch.ones(batch_size, 1).to(device)
        fake_labels = torch.zeros(batch_size, 1).to(device)

        # Train Discriminator
        optimizer_d.zero_grad()
        real_outputs = discriminator(high_quality)
        fake_audio = generator(low_quality)
        fake_outputs = discriminator(fake_audio.detach())
        d_loss_real = criterion(real_outputs, real_labels)
        d_loss_fake = criterion(fake_outputs, fake_labels)
        d_loss = (d_loss_real + d_loss_fake) * 0.5
        d_loss.backward()
        optimizer_d.step()

        # Train Generator
        optimizer_g.zero_grad()
        fake_outputs = discriminator(fake_audio)
        g_loss = criterion(fake_outputs, real_labels)
        g_loss.backward()
        optimizer_g.step()
        original = high_quality
        crap_audio = fake_audio

    if epoch % 10 == 0:
        print(crap_audio.size())
        torchaudio.save(f"./epoch-{epoch}-audio.wav", crap_audio[0].cpu(), 44100)
        torchaudio.save(f"./epoch-{epoch}-audio-orig.wav", original[0].cpu(), 44100)

    print(f'Epoch [{epoch+1}/{num_epochs}]')

torch.save(generator.state_dict(), "generator.pt")
torch.save(discriminator.state_dict(), "discriminator.pt")

print("Training complete!")
